@@ -47,7 +47,6 @@
 #include <assert.h>
 extern SourceReader	gReader;
 extern int  gMaxNameSize;
-extern bool gPatternEvalMode;
 extern bool	gSimpleNames;
 extern bool gSimplifyDiagrams;
 // History
@@ -81,13 +80,13 @@ static Tree		evalPattern(Tree pattern, Tree env);
 
 static Tree 	patternSimplification (Tree pattern);
 static bool 	isBoxNumeric (Tree in, Tree& out);
-static Tree 	replaceBoxNumeric (Tree exp);
-
 
 static Tree 	vec2list(const vector<Tree>& v);
 static void 	list2vec(Tree l, vector<Tree>& v);
 static Tree 	listn (int n, Tree e);
 
+static Tree     boxSimplification(Tree box);
+
 // Public Interface
 //----------------------
 
@@ -101,7 +100,13 @@ static Tree 	listn (int n, Tree e);
  */
 Tree evalprocess (Tree eqlist)
 {
-	return a2sb(eval(boxIdent("process"), nil, pushMultiClosureDefs(eqlist, nil, nil)));
+    Tree b = a2sb(eval(boxIdent("process"), nil, pushMultiClosureDefs(eqlist, nil, nil)));
+
+    if (gSimplifyDiagrams) {
+        b = boxSimplification(b);
+    }
+
+    return b;
 }
 
 
@@ -211,14 +216,7 @@ static Tree real_a2sb(Tree exp)
             if (b != m) modified=true;
 		}
         Tree r = (modified) ? CTree::make(exp->node(), B) : exp;
-        if (gSimplifyDiagrams) {
-            return replaceBoxNumeric(r);
-        } else {
-            return r;
-        }
-/*
-        if (modified) {
-		return replaceBoxNumeric(CTree::make(exp->node(), exp->arity(), B));*/
+        return r;
 	}
 }
 
@@ -252,14 +250,14 @@ static loopDetector LD(1024, 1);
 static Node EVALPROPERTY(symbol("EvalProperty"));
 
 /**
- * set the type annotation of sig
- * @param sig the signal we want to type
- * @param t the type of the signal
+ * set the value of box in the environment env
+ * @param box the block diagram we have evaluated
+ * @param env the evaluation environment
+ * @param value the evaluated block diagram
  */
 void setEvalProperty(Tree box, Tree env, Tree value)
 {
-	//cerr << "setSigType(" << *sig << ", " << t << ")" << endl;
-	setProperty(box, tree(EVALPROPERTY,env), value);
+    setProperty(box, tree(EVALPROPERTY,env), value);
 }
 
 
@@ -284,11 +282,10 @@ static Tree eval (Tree exp, Tree visited, Tree localValEnv)
 	LD.detect(cons(exp,localValEnv));
 	
 	if (!getEvalProperty(exp, localValEnv, result)) {
+        //cerr << "ENTER eval("<< *exp << ") with env " << *localValEnv << endl;
 		result = realeval(exp, visited, localValEnv);
 		setEvalProperty(exp, localValEnv, result);
-		
-		
-		
+        //cerr << "EXIT eval(" << *exp << ") IS " << *result << " with env " << *localValEnv << endl;
 		if (getDefNameProperty(exp, id)) {
 			setDefNameProperty(result, id);		// propagate definition name property 
 		}
@@ -319,7 +316,7 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 	Tree	id;
 
 	//cerr << "EVAL " << *exp << " (visited : " << *visited << ")" << endl;
-	//cerr << "REALEVAL of " << boxpp(exp) << endl;
+    //cerr << "REALEVAL of " << *exp << endl;
 	
 	xtended* xt = (xtended*) getUserData(exp);
 
@@ -483,8 +480,8 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 		return eval(body, visited, pushMultiClosureDefs(ldef, visited, localValEnv));
 	
 	} else if (isBoxAppl(exp, fun, arg)) {
-		return applyList(	eval(fun, visited, localValEnv),
-							revEvalList(arg, visited, localValEnv) );
+        return applyList( eval(fun, visited, localValEnv),
+						  revEvalList(arg, visited, localValEnv) );
 
     } else if (isBoxAbstr(exp)) {
         // it is an abstraction : return a closure
@@ -538,7 +535,7 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 	//---------------------------
 	
 	} else if (isBoxCase(exp, rules)) {
-		return evalCase(rules, localValEnv);
+        return evalCase(rules, localValEnv);
 
 	} else if (isBoxPatternVar(exp, id)) {
 		return exp;
@@ -583,35 +580,6 @@ bool getNumericProperty(Tree t, Tree& num)
 	return getProperty(t, NUMERICPROPERTY, num);
 }
 
-
-static Tree replaceBoxNumeric (Tree exp)
-{
-	int 	numInputs, numOutputs;
-	double 	x; 
-	int		i;
-	Tree	out;
-	
-	if (isBoxInt(exp, &i) || isBoxReal(exp, &x)) {
-		return exp;
-	} else if (getNumericProperty(exp, out)) {
-		return out;
-	} else {
-		if ( getBoxType(exp, &numInputs, &numOutputs) && (numInputs == 0) && (numOutputs == 1) ) {
-			// potential numerical expression
-			Tree lsignals = boxPropagateSig(nil, exp , makeSigInputList(numInputs) );
-			assert(isList(lsignals));
-			Tree res = simplify(hd(lsignals));
-			if (isSigReal(res, &x))			out = boxReal(x);
-			else if (isSigInt(res, &i))		out = boxInt(i);
-			else out =  exp;
-		} else {
-			out = exp;
-		}
-		setNumericProperty(exp,out);
-		return out;
-	}
-}
-
 /**
  * Simplify a block-diagram pattern by computing its numerical sub-expressions
  * \param pattern an evaluated block-diagram
@@ -635,31 +603,31 @@ Tree simplifyPattern (Tree value)
 
 static bool isBoxNumeric (Tree in, Tree& out)
 {
-	int 	numInputs, numOutputs;
-	double 	x; 
-	int		i;
-	Tree 	v;
-	
-	if (isBoxInt(in, &i) || isBoxReal(in, &x)) {
-		out = in;
-		return true;
-	} else {
-		v = a2sb(in);
-		if ( getBoxType(v, &numInputs, &numOutputs) && (numInputs == 0) && (numOutputs == 1) ) {
-			// potential numerical expression
-			Tree lsignals = boxPropagateSig(nil, v , makeSigInputList(numInputs) );
-			Tree res = simplify(hd(lsignals));
-			if (isSigReal(res, &x)) 	{
-			out = boxReal(x);
-			return true;
-			}
-			if (isSigInt(res, &i))  	{
-			out = boxInt(i);
-			return true;
-			}
-		} 
-		return false;
-	}
+    int 	numInputs, numOutputs;
+    double 	x;
+    int		i;
+    Tree 	v;
+
+    if (isBoxInt(in, &i) || isBoxReal(in, &x)) {
+        out = in;
+        return true;
+    } else {
+        v = a2sb(in);
+        if ( getBoxType(v, &numInputs, &numOutputs) && (numInputs == 0) && (numOutputs == 1) ) {
+            // potential numerical expression
+            Tree lsignals = boxPropagateSig(nil, v , makeSigInputList(numInputs) );
+            Tree res = simplify(hd(lsignals));
+            if (isSigReal(res, &x)) 	{
+            out = boxReal(x);
+            return true;
+            }
+            if (isSigInt(res, &i))  	{
+            out = boxInt(i);
+            return true;
+            }
+        }
+        return false;
+    }
 }
 
 static Tree patternSimplification (Tree pattern)
@@ -694,7 +662,7 @@ static Tree patternSimplification (Tree pattern)
  */
 static double eval2double (Tree exp, Tree visited, Tree localValEnv)
 {
-	Tree diagram = eval(exp, visited, localValEnv);
+    Tree diagram = a2sb(eval(exp, visited, localValEnv)); // pour getBoxType
 	int numInputs, numOutputs;
 	getBoxType(diagram, &numInputs, &numOutputs);
 	if ( (numInputs > 0) || (numOutputs != 1) ) {
@@ -723,7 +691,7 @@ static double eval2double (Tree exp, Tree visited, Tree localValEnv)
  */
 static int eval2int (Tree exp, Tree visited, Tree localValEnv)
 {
-	Tree diagram = eval(exp, visited, localValEnv);
+    Tree diagram = a2sb(eval(exp, visited, localValEnv));   // pour getBoxType()
 	int numInputs, numOutputs;
 	getBoxType(diagram, &numInputs, &numOutputs);
 	if ( (numInputs > 0) || (numOutputs != 1) ) {
@@ -822,7 +790,7 @@ static const char * evalLabel (const char* label, Tree visited, Tree localValEnv
  * Iterate a parallel construction
  *
  * Iterate a parallel construction such that :
- * par(i,10,E) --> E(i<-0),E(i<-1),...,E(i<-9)
+ * par(i,10,E) --> E(i<-0),(E(i<-1),...,E(i<-9))
  * @param id the formal parameter of the iteration
  * @param num the number of iterartions
  * @param body the body expression of the iteration
@@ -833,14 +801,14 @@ static const char * evalLabel (const char* label, Tree visited, Tree localValEnv
  */
 static Tree iteratePar (Tree id, int num, Tree body, Tree visited, Tree localValEnv)
 {
-	assert (num>0);
+    assert (num>0);
 
-	Tree res = eval(body, visited, pushValueDef(id, tree(0), localValEnv));
-	for (int i = 1; i < num; i++) {
-		res = boxPar(res, eval(body, visited, pushValueDef(id, tree(i), localValEnv)));
-	}
+    Tree res = eval(body, visited, pushValueDef(id, tree(num-1), localValEnv));
+    for (int i = num-2; i >= 0; i--) {
+        res = boxPar(eval(body, visited, pushValueDef(id, tree(i), localValEnv)), res);
+    }
 
-	return res;
+    return res;
 }
 
 
@@ -849,7 +817,7 @@ static Tree iteratePar (Tree id, int num, Tree body, Tree visited, Tree localVal
  * Iterate a sequential construction
  *
  * Iterate a sequential construction such that :
- * seq(i,10,E) --> E(i<-0):E(i<-1):...:E(i<-9)
+ * seq(i,10,E) --> E(i<-0):(E(i<-1):...:E(i<-9))
  * @param id the formal parameter of the iteration
  * @param num the number of iterartions
  * @param body the body expression of the iteration
@@ -861,11 +829,10 @@ static Tree iterateSeq (Tree id, int num, Tree body, Tree visited, Tree localVal
 {
 	assert (num>0);
 
-	Tree res = eval(body, visited, pushValueDef(id, tree(0), localValEnv));
-
-	for (int i = 1; i < num; i++) {
-		res = boxSeq(res, eval(body, visited, pushValueDef(id, tree(i), localValEnv)));
-	}
+    Tree res = eval(body, visited, pushValueDef(id, tree(num-1), localValEnv));
+    for (int i = num-2; i >= 0; i--) {
+        res = boxSeq(eval(body, visited, pushValueDef(id, tree(i), localValEnv)), res);
+    }
 
 	return res;
 }
@@ -942,7 +909,8 @@ static bool boxlistOutputs(Tree boxlist, int* outputs)
     *outputs = 0;
     while (!isNil(boxlist))
     {
-    	if (getBoxType(hd(boxlist), &ins, &outs)) {
+        Tree b = a2sb(hd(boxlist)); // for getBoxType, suppose list of evaluated boxes
+        if (getBoxType(b, &ins, &outs)) {
             *outputs += outs;
       	} else {
       		// arbitrary output arity set to 1
@@ -1007,6 +975,8 @@ static Tree applyList (Tree fun, Tree larg)
 
 	prim2	p2;
 
+    //cerr << "applyList (" << *fun << ", " << *larg << ")" << endl;
+
 	if (isNil(larg)) return fun;
 
 	if (isBoxError(fun) || isBoxError(larg)) {
@@ -1019,7 +989,9 @@ static Tree applyList (Tree fun, Tree larg)
 		vector<Tree>	envVect;
 		
 		list2vec(envList, envVect);
+        //cerr << "applyList/apply_pattern_matcher(" << automat << "," << state << "," << *hd(larg) << ")" << endl;
 		state2 = apply_pattern_matcher(automat, state, hd(larg), result, envVect);
+        //cerr << "state2 = " << state2 << "; result = " << *result << endl;
 		if (state2 >= 0 && isNil(result)) {
 			// we need to continue the pattern matching
 			return applyList(
@@ -1047,7 +1019,9 @@ static Tree applyList (Tree fun, Tree larg)
          int ins, outs;
          
          // check arity of function
-         if (!getBoxType(fun, &ins, &outs)) {
+         Tree efun = a2sb(fun);
+         //cerr << "TRACEPOINT 1 : " << boxpp(efun) << endl;
+         if (!getBoxType(efun, &ins, &outs)) { // on laisse comme ca pour le moment
          	// we can't determine the input arity of the expression
          	// hope for the best
          	return boxSeq(larg2par(larg), fun);
@@ -1123,11 +1097,15 @@ static Tree applyList (Tree fun, Tree larg)
  */
 static Tree revEvalList (Tree lexp, Tree visited, Tree localValEnv)
 {
-	Tree result = nil;
-	while (!isNil(lexp)) {
+    Tree result = nil;
+    //Tree lexp_orig = lexp;
+    //cerr << "ENTER revEvalList(" << *lexp_orig << ", env:" << *localValEnv << ")" << endl;
+    while (!isNil(lexp)) {
 		result = cons(eval(hd(lexp), visited, localValEnv), result);
 		lexp = tl(lexp);
 	}
+
+    //cerr << "EXIT revEvalList(" << *lexp_orig << ", env:" << *localValEnv << ") -> " << *result << endl;
 	return result;
 }
 
@@ -1175,13 +1153,12 @@ static Tree evalIdDef(Tree id, Tree visited, Tree lenv)
 
 	// check that the definition exists
 	if (isNil(lenv)) {
-		if (gPatternEvalMode) return boxPatternVar(id);
-		cerr << "undefined symbol " << *id << endl;
+        cerr << "undefined symbol " << *id << endl;
 		evalerror(getDefFileProp(id), getDefLineProp(id), "undefined symbol ", id);
 		exit(1);
-//		return id;
 	}
 
+    //cerr << "Id definition is " << *def << endl;
 	// check that it is not a recursive definition
 	Tree p = cons(id,lenv);
 	// set the definition name property
@@ -1239,8 +1216,8 @@ static Tree	evalCase(Tree rules, Tree env)
 	Tree pm;
 	if (!getPMProperty(rules, env, pm)) {
 		Automaton*	a = make_pattern_matcher(evalRuleList(rules, env));
-		pm = boxPatternMatcher(a, 0, listn(len(rules), env), rules, nil);
-		setPMProperty(rules, env, pm);
+        pm = boxPatternMatcher(a, 0, listn(len(rules), pushEnvBarrier(env)), rules, nil);
+        setPMProperty(rules, env, pm);
 	}
 	return pm;
 }		
@@ -1251,6 +1228,7 @@ static Tree	evalCase(Tree rules, Tree env)
  */
 static Tree	evalRuleList(Tree rules, Tree env)
 {
+    //cerr << "evalRuleList "<< *rules << " in " << *env << endl;
 	if (isNil(rules)) return nil;
 	else return cons(evalRule(hd(rules), env), evalRuleList(tl(rules), env));
 }
@@ -1261,6 +1239,7 @@ static Tree	evalRuleList(Tree rules, Tree env)
  */
 static Tree	evalRule(Tree rule, Tree env)
 {
+    //cerr << "evalRule "<< *rule << " in " << *env << endl;
 	return cons(evalPatternList(left(rule), env), right(rule));
 }
 
@@ -1280,16 +1259,13 @@ static Tree	evalPatternList(Tree patterns, Tree env)
 
 
 /**
- * Evaluates a pattern using a special mode
- * so that free variables are wrapped into a boxPatternVar 
+ * Evaluates a pattern and simplify it to numerical value 
+ * if possible 
  */
 static Tree	evalPattern(Tree pattern, Tree env)
 {
-	bool saveMode = gPatternEvalMode;
-	gPatternEvalMode = true;
-	Tree p = eval(pattern, nil, env);
-	gPatternEvalMode = saveMode;
-	return patternSimplification(p);
+    Tree p = eval(pattern, nil, env);
+    return patternSimplification(p);
 }
 
 
@@ -1309,3 +1285,255 @@ static Tree vec2list(const vector<Tree>& v)
 	while (n--) { l = cons(v[n],l); }
 	return l;
 }
+
+
+
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////
+// further simplification : replace bloc-diagrams that denote constant number by this number
+/////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+static property<Tree> SimplifiedBoxProperty;
+static Tree numericBoxSimplification(Tree box);
+static Tree insideBoxSimplification (Tree box);
+
+/**
+ * boxSimplification(box) : simplify a block-diagram by replacing expressions
+ * denoting a constant number by this number.
+ */
+Tree boxSimplification (Tree box)
+{
+    Tree    simplified;
+
+    if (SimplifiedBoxProperty.get(box,simplified)) {
+
+        return simplified;
+
+    } else {
+
+        simplified = numericBoxSimplification(box);
+
+        // transferts name property if any
+        Tree name; if (getDefNameProperty(box, name)) setDefNameProperty(simplified, name);
+
+        // attach simplified expression as a property of original box
+        SimplifiedBoxProperty.set(box,simplified);
+
+        return simplified;
+    }
+}
+
+/**
+ * Try to do a numeric simplification of a block-diagram
+ */
+Tree numericBoxSimplification(Tree box)
+{
+    int     ins, outs;
+    Tree    result;
+    int     i;
+    double  x;
+
+    if ( ! getBoxType(box, &ins, &outs)) {
+        cout << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", Can't compute the box type of : " << *box << endl;
+        exit(1);
+    }
+
+    if (ins==0 && outs==1) {
+        // this box can potentially denote a number
+        if (isBoxInt(box, &i) || isBoxReal(box, &x)) {
+           result = box;
+        } else {
+            // propagate signals to discover if it simplifies to a number
+            int     i;
+            double  x;
+            Tree    lsignals = boxPropagateSig(nil, box , makeSigInputList(0));
+            Tree    s = simplify(hd(lsignals));
+
+            if (isSigReal(s, &x)) 	{
+                result = boxReal(x);
+            } else if (isSigInt(s, &i))  	{
+                result = boxInt(i);
+            } else {
+                result = insideBoxSimplification(box);
+            }
+        }
+    } else {
+        // this box can't denote a number
+        result = insideBoxSimplification(box);
+    }
+    return result;
+}
+
+/**
+ *  Simplify inside a block-diagram : S[A*B] => S[A]*S[B]
+ */
+Tree insideBoxSimplification (Tree box)
+{
+    int		i;
+    double	r;
+    prim0	p0;
+    prim1	p1;
+    prim2	p2;
+    prim3	p3;
+    prim4	p4;
+    prim5	p5;
+
+    Tree	t1, t2, ff, label, cur, min, max, step, type, name, file, slot, body;
+
+
+    xtended* xt = (xtended*)getUserData(box);
+
+    // Extended Primitives
+
+    if (xt)	{
+        return box;
+    }
+
+    // Numbers and Constants
+
+    else if (isBoxInt(box, &i)) 	{
+        return box;
+    }
+    else if (isBoxReal(box, &r)) 	{
+        return box;
+    }
+
+    else if (isBoxFConst(box, type, name, file))    {
+        return box;
+    }
+
+    else if (isBoxFVar(box, type, name, file))    {
+        return box;
+    }
+
+    // Wire and Cut
+
+    else if (isBoxCut(box)) 				{
+        return box;
+    }
+
+    else if (isBoxWire(box)) 				{
+        return box;
+    }
+
+    // Primitives
+
+    else if (isBoxPrim0(box, &p0)) 			{
+        return box;
+    }
+
+    else if (isBoxPrim1(box, &p1)) 			{
+        return box;
+    }
+
+    else if (isBoxPrim2(box, &p2)) 				{
+        return box;
+    }
+
+    else if (isBoxPrim3(box, &p3)) 				{
+        return box;
+    }
+
+    else if (isBoxPrim4(box, &p4)) 				{
+        return box;
+    }
+
+    else if (isBoxPrim5(box, &p5)) 				{
+        return box;
+    }
+
+    else if (isBoxFFun(box, ff)) 				{
+        return box;
+    }
+
+    // User Interface Widgets
+
+    else if (isBoxButton(box, label)) 	{
+        return box;
+    }
+
+    else if (isBoxCheckbox(box, label)) 	{
+        return box;
+    }
+
+    else if (isBoxVSlider(box, label, cur, min, max, step)) 	{
+        return box;
+    }
+
+    else if (isBoxHSlider(box, label, cur, min, max, step)) 	{
+        return box;
+    }
+
+    else if (isBoxNumEntry(box, label, cur, min, max, step)) 	{
+        return box;
+    }
+
+    else if (isBoxVBargraph(box, label, min, max)) 	{
+        return box;
+    }
+
+    else if (isBoxHBargraph(box, label, min, max)) 	{
+        return box;
+    }
+
+    // User Interface Groups
+
+    else if (isBoxVGroup(box, label, t1)) 	{
+        return boxVGroup(label, boxSimplification(t1));
+    }
+
+    else if (isBoxHGroup(box, label, t1)) 	{
+        return boxHGroup(label, boxSimplification(t1));
+    }
+
+    else if (isBoxTGroup(box, label, t1)) 	{
+        return boxTGroup(label, boxSimplification(t1));
+    }
+
+    // Slots and Symbolic Boxes
+
+    else if (isBoxSlot(box)) 				{
+        return box;;
+    }
+
+    else if (isBoxSymbolic(box, slot, body)){
+
+        Tree b = boxSimplification(body);
+        return boxSymbolic(slot,b);
+    }
+
+    // Block Diagram Composition Algebra
+
+    else if (isBoxSeq(box, t1, t2)) 	{
+        Tree s1 = boxSimplification(t1);
+        Tree s2 = boxSimplification(t2);
+        return boxSeq(s1,s2);
+    }
+
+    else if (isBoxPar(box, t1, t2)) 	{
+        Tree s1 = boxSimplification(t1);
+        Tree s2 = boxSimplification(t2);
+        return boxPar(s1,s2);
+    }
+
+    else if (isBoxSplit(box, t1, t2)) 	{
+        Tree s1 = boxSimplification(t1);
+        Tree s2 = boxSimplification(t2);
+        return boxSplit(s1,s2);
+    }
+
+    else if (isBoxMerge(box, t1, t2)) 	{
+        Tree s1 = boxSimplification(t1);
+        Tree s2 = boxSimplification(t2);
+        return boxMerge(s1,s2);
+    }
+    else if (isBoxRec(box, t1, t2)) 	{
+        Tree s1 = boxSimplification(t1);
+        Tree s2 = boxSimplification(t2);
+        return boxRec(s1,s2);
+    }
+
+    cout << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", unrecognised box expression : " << *box << endl;
+    exit(1);
+    return 0;
+}
