@@ -18,7 +18,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
-#define FAUSTVERSION "0.9.24"
+#define FAUSTVERSION "0.9.43"
 
 #include <stdio.h>
 #include <string.h>
@@ -112,7 +112,8 @@ Tree			gExpandedDefList;
 bool			gHelpSwitch 	= false;
 bool			gVersionSwitch 	= false;
 bool            gDetailsSwitch  = false;
-bool            gShadowBlur     = false;
+bool            gDrawSignals    = false;
+bool            gShadowBlur     = false;	// note: svg2pdf doesn't like the blur filter
 bool            gGraphSwitch 	= false;
 bool            gDrawPSSwitch 	= false;
 bool            gDrawSVGSwitch 	= false;
@@ -139,10 +140,12 @@ int             gVecSize        = 32;
 int             gVectorLoopVariant = 0;
 
 bool            gOpenMPSwitch   = false;
+bool            gOpenMPLoop     = false;
 bool            gSchedulerSwitch   = false;
 bool			gGroupTaskSwitch= false;
 
 bool            gUIMacroSwitch  = false;
+bool            gDumpNorm       = false;
 
 int             gTimeout        = 0;            // time out to abort compiler
 
@@ -150,6 +153,8 @@ int             gFloatSize = 1;
 
 bool			gPrintFileListSwitch = false;
 
+string			gClassName		= "mydsp";
+
 //-- command line tools
 
 static bool isCmd(const char* cmd, const char* kw1)
@@ -200,6 +205,10 @@ bool process_cmdline(int argc, char* argv[])
             gGraphSwitch = true;
             i += 1;
 
+        } else if (isCmd(argv[i], "-sg", "--signal-graph")) {
+            gDrawSignals = true;
+            i += 1;
+
         } else if (isCmd(argv[i], "-blur", "--shadow-blur")) {
             gShadowBlur = true;
             i += 1;
@@ -262,7 +271,11 @@ bool process_cmdline(int argc, char* argv[])
 
         } else if (isCmd(argv[i], "-omp", "--openMP")) {
             gOpenMPSwitch = true;
-         	i += 1;
+            i += 1;
+
+        } else if (isCmd(argv[i], "-pl", "--par-loop")) {
+            gOpenMPLoop = true;
+            i += 1;
 
         } else if (isCmd(argv[i], "-sch", "--scheduler")) {
 			gSchedulerSwitch = true;
@@ -308,8 +321,16 @@ bool process_cmdline(int argc, char* argv[])
         } else if (isCmd(argv[i], "-flist", "--file-list")) {
             gPrintFileListSwitch = true;
             i += 1;
-			
-		} else if (argv[i][0] != '-') {
+
+        } else if (isCmd(argv[i], "-norm", "--normalized-form")) {
+            gDumpNorm = true;
+            i += 1;
+
+		} else if (isCmd(argv[i], "-cn", "--class-name")) {
+			gClassName = argv[i+1];
+			i += 2;
+
+        } else if (argv[i][0] != '-') {
 			if (check_file(argv[i])) {
 				gInputFiles.push_back(argv[i]);
 			}
@@ -339,7 +360,7 @@ bool process_cmdline(int argc, char* argv[])
 void printversion()
 {
 	cout << "FAUST, DSP to C++ compiler, Version " << FAUSTVERSION << "\n";
-	cout << "Copyright (C) 2002-2010, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
+	cout << "Copyright (C) 2002-2011, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
 }
 
 
@@ -356,6 +377,7 @@ void printhelp()
 	cout << "-v \t\tprint compiler --version information\n";
 	cout << "-d \t\tprint compilation --details\n";
     cout << "-tg \t\tprint the internal --task-graph in dot format file\n";
+    cout << "-sg \t\tprint the internal --signal-graph in dot format file\n";
     cout << "-ps \t\tprint block-diagram --postscript file\n";
     cout << "-svg \tprint block-diagram --svg file\n";
     cout << "-mdoc \tprint --mathdoc of a Faust program in LaTeX format in a -mdoc directory\n";
@@ -373,11 +395,13 @@ void printhelp()
 	cout << "-lt \t\tgenerate --less-temporaries in compiling delays\n";
 	cout << "-mcd <n> \t--max-copy-delay <n> threshold between copy and ring buffer implementation (default 16 samples)\n";
 	cout << "-a <file> \tC++ architecture file\n";
-    cout << "-o <file> \tC++ output file\n";
+	cout << "-cn <name> \t--class-name <name> specify the name of the dsp class to be used instead of mydsp \n";
+	cout << "-o <file> \tC++ output file\n";
     cout << "-vec    \t--vectorize generate easier to vectorize code\n";
     cout << "-vs <n> \t--vec-size <n> size of the vector (default 32 samples)\n";
     cout << "-lv <n> \t--loop-variant [0:fastest (default), 1:simple] \n";
     cout << "-omp    \t--openMP generate OpenMP pragmas, activates --vectorize option\n";
+    cout << "-pl     \t--par-loop generate parallel loops in --openMP mode\n";
     cout << "-sch    \t--scheduler generate tasks and use a Work Stealing scheduler, activates --vectorize option\n";
 	cout << "-dfs    \t--deepFirstScheduling schedule vector loops in deep first order\n";
     cout << "-g    \t\t--groupTasks group single-threaded sequential tasks together when -omp or -sch is used\n";
@@ -386,6 +410,7 @@ void printhelp()
     cout << "-double \tuse --double-precision-floats for internal computations\n";
     cout << "-quad \t\tuse --quad-precision-floats for internal computations\n";
     cout << "-flist \t\tuse --file-list used to eval process\n";
+    cout << "-norm \t\t--normalized-form prints signals in normalized form and exits\n";
 
 	cout << "\nexample :\n";
 	cout << "---------\n";
@@ -430,12 +455,6 @@ void printheader(ostream& dst)
 *****************************************************************/
 
 
-static string dirname(const string& path)
-{
-    char s[1024];
-    strncpy(s, path.c_str(), 1024);
-    return string(dirname(s));
-}
 
 /**
  * transform a filename "faust/example/noise.dsp" into
@@ -463,10 +482,10 @@ static void initFaustDirectories()
 {
     char s[1024];
     getFaustPathname(s, 1024);
-    dirname(s);
-    gFaustDirectory = s;
-	gFaustSuperDirectory = dirname(gFaustDirectory);
-	gFaustSuperSuperDirectory = dirname(gFaustSuperDirectory);
+
+    gFaustDirectory = filedirname(s);
+    gFaustSuperDirectory = filedirname(gFaustDirectory);
+    gFaustSuperSuperDirectory = filedirname(gFaustSuperDirectory);
     if (gInputFiles.empty()) {
         gMasterDocument = "Unknown";
         gMasterDirectory = ".";
@@ -474,7 +493,7 @@ static void initFaustDirectories()
 		gDocName = "faustdoc";
     } else {
         gMasterDocument = *gInputFiles.begin();
-        gMasterDirectory = dirname(gMasterDocument);
+        gMasterDirectory = filedirname(gMasterDocument);
 		gMasterName = fxname(gMasterDocument);
 		gDocName = fxname(gMasterDocument);
     }
@@ -534,7 +553,8 @@ int main (int argc, char* argv[])
 	startTiming("evaluation");
 
 
-	Tree process = evalprocess(gExpandedDefList);
+    Tree process = evalprocess(gExpandedDefList);
+
 	if (gErrorCount > 0) {
        // cerr << "Total of " << gErrorCount << " errors during evaluation of : process = " << boxpp(process) << ";\n";
         cerr << "Total of " << gErrorCount << " errors during the compilation of  " << gMasterDocument << ";\n";
@@ -600,9 +620,9 @@ int main (int argc, char* argv[])
 	startTiming("compilation");
 
 	Compiler* C;
-    if (gSchedulerSwitch)   C = new SchedulerCompiler("mydsp", "dsp", numInputs, numOutputs);
-    else if (gVectorSwitch) C = new VectorCompiler("mydsp", "dsp", numInputs, numOutputs);
-	else                    C = new ScalarCompiler("mydsp", "dsp", numInputs, numOutputs);
+	if (gSchedulerSwitch)   C = new SchedulerCompiler(gClassName, "dsp", numInputs, numOutputs);
+	else if (gVectorSwitch) C = new VectorCompiler(gClassName, "dsp", numInputs, numOutputs);
+	else                    C = new ScalarCompiler(gClassName, "dsp", numInputs, numOutputs);
 
 	if (gPrintXMLSwitch) C->setDescription(new Description());
 	if (gPrintDocSwitch) C->setDescription(new Description());
@@ -671,8 +691,8 @@ int main (int argc, char* argv[])
 			C->getClass()->printIncludeFile(*dst);
             C->getClass()->printAdditionalCode(*dst);
 
-			streamCopyUntil(*enrobage, *dst, "<<includeIntrinsic>>");
-            
+            streamCopyUntil(*enrobage, *dst, "<<includeIntrinsic>>");
+
 // 			if ( gVectorSwitch && (intrinsic = open_arch_stream("intrinsic.hh")) ) {
 // 				streamCopyUntilEnd(*intrinsic, *dst);
 // 			}
